/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

/************************************************************************/
/* 0. common stuff                                                      */
/************************************************************************/

/* include the file with the description of specific feature support */
#include "_rl_cfg.h"


#if defined CPU_ATMEGA16 || defined CPU_ATMEGA168

#else

#error "no supported target hardware selected"

#endif


#define   REBOOT_LED_ON        ( PORTC = PORTC |  (1<<PORTC6) )
#define   REBOOT_LED_OFF       ( PORTC = PORTC & ~(1<<PORTC6) )

/* a 16 bit value (e.g. sensor value) also accessing by high/low byte */
typedef union
{
  uint16_t u16_value;
  struct
  {
    uint8_t l_value;
    uint8_t h_value;
  } as;
} VAR_U16;



/************************************************************************/
/* 1. RV3029 real time clock                                            */
/************************************************************************/

/* 1.1. register definitions                                            */
/* control page */
#define RV3029_REG_CON_1          0x00
#define RV3029_REG_CON_INT        0x01
#define RV3029_REG_CON_INT_FLAG   0x02
#define RV3029_REG_CON_STATUS     0x03
#define RV3029_REG_CON_RESET      0x04
/* clock page */
#define RV3029_REG_CLK_SECONDS    0x08
#define RV3029_REG_CLK_MINUTES    0x09
#define RV3029_REG_CLK_HOURS      0x0A
#define RV3029_REG_CLK_DAYS       0x0B
#define RV3029_REG_CLK_WEEKDAYS   0x0C
#define RV3029_REG_CLK_MONTHS     0x0D
#define RV3029_REG_CLK_YEARS      0x0E
/* alarm page */
#define RV3029_REG_ALM_SECONDS    0x10
#define RV3029_REG_ALM_MINUTES    0x11
#define RV3029_REG_ALM_HOURS      0x12
#define RV3029_REG_ALM_DAYS       0x13
#define RV3029_REG_ALM_WEEKDAYS   0x14
#define RV3029_REG_ALM_MONTHS     0x15
#define RV3029_REG_ALM_YEARS      0x16
/* timer page */
#define RV3029_REG_TIM_LOW        0x18
#define RV3029_REG_TIM_HIGH       0x19
/* temperature page */
#define RV3029_REG_TEMP           0x20
/* eeprom page */
#define RV3029_REG_EEP_01         0x28
#define RV3029_REG_EEP_02         0x29
/* eeprom control page */
#define RV3029_REG_EEPCTL         0x30
#define RV3029_REG_EEPCTL_OFFSET  0x31
#define RV3029_REG_EEPCTL_COEF    0x32
#define RV3029_REG_EEPCTL_XTAL_T0 0x33
/* user ram page */
#define RV3029_REG_RAM_01         0x38
#define RV3029_REG_RAM_02         0x39
#define RV3029_REG_RAM_03         0x3A
#define RV3029_REG_RAM_04         0x3B
#define RV3029_REG_RAM_05         0x3C
#define RV3029_REG_RAM_06         0x3D
#define RV3029_REG_RAM_07         0x3E
#define RV3029_REG_RAM_08         0x3F

#define RV3029_WATCH_EL_WEEKDAY   0x00
#define RV3029_WATCH_EL_DAY       0x01
#define RV3029_WATCH_EL_MONTH     0x02
#define RV3029_WATCH_EL_YEAR      0x03
#define RV3029_WATCH_EL_HOUR      0x04
#define RV3029_WATCH_EL_MINUTE    0x05
#define RV3029_WATCH_EL_SECOND    0x06
#define RV3029_WATCH_EL_CNT       0x07


/* 1.2. routines                                                        */

extern void     rv3029_init                     (uint8_t reset_flag);
extern void     rv3029_set_time                 (uint8_t * dt_field);
extern void     rv3029_read_time                (void);
extern void     rv3029_write_temperature        (uint8_t item);
extern void     rv3029_set_hour                 (uint8_t correction);
extern void     rv3029_get_current_hour_minute  (uint8_t * hour, uint8_t * minute);


/************************************************************************/
/* 2. MCP23017                                                          */
/************************************************************************/

/* 2.1. register definitions - banking is set                           */
/*      second bank located at offset 0x10                              */

#define MCP23017_IODIR 				0x00
#define MCP23017_IPOL  				0x01
#define MCP23017_GPINTEN 			0x02
#define MCP23017_DEFVAL 			0x03
#define MCP23017_INTCON 			0x04
#define MCP23017_IOCON 				0x05
#define MCP23017_GPPU 				0x06
#define MCP23017_INTF 				0x07
#define MCP23017_INTCAP 			0x08
#define MCP23017_GPIO 				0x09
#define MCP23017_OLAT 				0x0A

#define MCP23017_KP_BANK          0x00
#define MCP23017_LCD_BANK         0x10
#define MCP23017_WB_UP_BANK       0x00
#define MCP23017_WB_DOWN_BANK     0x10
#define MCP23017_MOTOR_DIR_BANK   0x00
#define MCP23017_MOTOR_PWR_BANK   0x10

/* 2.2. i2c addresses of the used mcp23017                              */

#define MCP23017_ADDRESS_HMI      (0x00<<4)     /* human machine interface: lcd display and keypad */
#define MCP23017_ADDRESS_WB       (0x02<<4)     /* wallbutton up-down keys                         */
#define MCP23017_ADDRESS_MOTOR    (0x04<<4)     /* motors of blinds (only RL_DESIGN_R0)            */

/* 2.3. routines                                                */

extern void 	  mcp23017_out              (uint8_t address_bank, uint8_t data);
extern uint8_t  mcp23017_in               (uint8_t address_bank);
extern void 	  mcp23017_write_register   (uint8_t address_bank_reg, uint8_t data);
extern void     mcp23017_init_for_input   (uint8_t address_bank);

/************************************************************************/
/* 3. Human machine interface (HMI)                                     */
/************************************************************************/

/* 3.1. defines                                         */

/* 3.2. types                                           */

/* 3.3. routines                                        */

extern void hmi_init            (void);
extern void hmi_pull_plug_check (void);


/************************************************************************/
/* 4. Keypad settings                                                   */
/************************************************************************/

/* 4.1. defines                                         */

#define KP_RIGHT          0x01
#define KP_LEFT           0x02
#define KP_CURSOR_LEFT    0x04
#define KP_CURSOR_DOWN    0x08
#define KP_CURSOR_MIDDLE  0x10
#define KP_CURSOR_UP      0x20
#define KP_CURSOR_RIGHT   0x40

#define WB_UP_INDEX            0     /* wall button up bank    */
#define WB_DOWN_INDEX          1     /* wall button down bank  */
#define KP_INDEX               2     /* keypad bank            */
#define KEY_BANK_MAX           3

/* 4.2. types                                                              */

typedef struct
{
  uint8_t input;
  uint8_t state[8];                   /* every bit of a byte supports a switch with its state -> 8 switches */
  uint8_t sum;                        /* bitlane: key 1..8 ha been pressed, longpressed, released */
  uint8_t released;                   /* bitlane: key 1..8 has been released     */
  uint8_t pressed;                    /* bitlane: key 1..8 has been pressed      */
  uint8_t longpressed;                /* bitlane: key 1..8 has been longpressed  */
} KEY_BANK;


/* 4.3. routines                                                 */

extern void    key_hmi_init     (void);
extern void    key_init         (void);
extern void    key_timer_10ms   (void);
extern void    key_evaluate     (void);

/************************************************************************/
/* 5. radio frequency support                                            */
/************************************************************************/


/* 5.1. defines                                                          */

#define RC_TELEGRAM_SIZE            3 /* 5 digit system code, 5 digit device code, 1 digit switch-off, 1 digit switch-on -- every digit consists of 2bit */

#define RC_BINARY_VALUE_ZERO            0x00
#define RC_BINARY_VALUE_FLOATING        0x01
#define RC_BINARY_VALUE_ONE             0x02
#define RC_BINARY_VALUE_INVALID         0x03

#define RC_TIME_2_TACT                (87*2)  /* the timer 0 is set to a base of 4us -> one tact is around 350us (87*4us) */
#define RC_TIME_TIMEOUT                (250)

#define RC_KEY_TIMEOUT                 9 /* base is 100ms -> timeout appears after no known keys are pressed on the remote (simulate a trigger for key release) */

/* every hs2262 digit consists of 2 bits as shown below */
#define RC_DIGIT_VALUE_FLOATING         0x01
#define RC_DIGIT_VALUE_ONE              0x03
#define RC_DIGIT_VALUE_ZERO             0x00
#define RC_DIGIT_VALUE_INVALID          0x02
#define RC_DIGIT_VALUE_MASK             0x03

#define RC_DIGIT_VALUE_NO               0xFF
#define RC_DIGIT_VALUE_OFF              0x04
#define RC_DIGIT_VALUE_OFF_FORCE        0x07    /* used for controlling an extender: driving down without minding the wallbutton state machine */
#define RC_DIGIT_VALUE_ON               0x01
#define RC_DIGIT_VALUE_ON_FORCE         0x0D    /* used for controlling an extender: driving up without minding the wallbutton state machine */
#define RC_DIGIT_VALUE_ON_OFF           0x00    /* both buttons of a couple (A..E) are pressed */
#define RC_DIGIT_VALUE_ON_OFF_MASK      0x0F

#define RC_TX_DEFAULT_REPETITION_CNT  14

#define RC_TX_QUEUE_SIZE              (RL_RC_TIMER_PAGES+(RL_COUNT_FULL-RL_COUNT)+2+1+1) /* 8 timer pages + extenders + 2 lightbow time + 1 testcode + 1 safety */

/* 5.2. strurtures                                                        */

typedef struct
{
  uint8_t byte_loc;     /* byte number of RC_TELEGRAM_SIZE that contains the desired system code digit                 */
  uint8_t byte_shift;   /* for how many digits is the desired system code to be shifted to the border of an unsigned8  */

} RC_CODE_LOCATION;

extern const RC_CODE_LOCATION rc_sys_code[5];
extern const RC_CODE_LOCATION rc_dev_code[5];

typedef struct
{
  uint8_t syscode;      /* syscode 1..4                 */
  uint8_t sys_devcode;  /* syscode 5 and devcode A..E   */

} RC_COMMAND;

/* 5.3. functions                                                        */

extern uint8_t  rc_get_syscode_digit_from_device      (uint8_t * rc_code, uint8_t sys_code_digit_nr);
extern void     rc_set_syscode_digit_to_device        (uint8_t * rc_code, uint8_t sys_code_digit_nr, uint8_t sys_code_value);
extern uint8_t  rc_get_devcode_digit_from_device      (uint8_t rc_code_02);
extern void     rc_set_devcode_digit_to_device        (uint8_t * rc_code_02, uint8_t sys_code_digit_nr);



extern void     rc_init                  (void);


extern void     rc_rx_init                  (void);
extern void     rc_rx_timeout               (void);
extern void     rc_rx_evaluate              (void);

extern void     rc_tx_init                  (void);
extern void     rc_tx_send_command          (RC_COMMAND * rc_command, uint8_t on_off_command, uint8_t repetition_cnt);
extern void     rc_tx_command_scheduler     (void);




/************************************************************************/
/* 6. menu (shown on lcd)                                               */
/************************************************************************/

/* 6.1. defines                                                          */

#define RL_N_BLIND_TIME_PAGES  24  /* the year is devided into 24 pages - every 15 days will be adaption to sunrise/sunset time        */

#define RL_RC_TIMER_PAGES       8  /* we have 8 pages where to set transmission command for the remote control  */


/* 6.2. matrix                                                          */

enum
{
  MENU_INDEX_N_BLIND,
  MENU_INDEX_D_BLIND,
  MENU_INDEX_MOTOR,
  MENU_INDEX_RC_TIMER,
  MENU_INDEX_DT,
  MENU_INDEX_ADV,
  MENU_INDEX_REBOOTS,
  MENU_INDEX_MISC,
  MENU_INDEX_CURR_VALUE,
  MENU_INDEX_MAIN,
  MENU_INDEX_MAX
} MENU_INDIZES;

#define MENU_ITEM_STACK_SIZE 19 /* menu rc_timer has the most items */
#define MENU_SELECTION_INIT       1
#define MENU_SELECTION_SRV        2

enum
{
  MENU_VAR_WEEKDAY,
  MENU_VAR_DAY,
  MENU_VAR_MONTH,
  MENU_VAR_YEAR,
  MENU_VAR_HOUR,
  MENU_VAR_MINUTE,
  MENU_VAR_SECOND,
  MENU_VAR_BOOLEAN,
  MENU_VAR_BOOLEAN_02,
  MENU_VAR_RL_CNT,
  MENU_VAR_RL_CNT_FULL,
  MENU_VAR_SENSOR,
  MENU_VAR_N_BLIND_PAGE,
  MENU_VAR_TIMER_PAGE,
  MENU_VAR_SHADOWING_SM,
  MENU_VAR_TEMPERATURE,
  MENU_VAR_RC_SYS,              /* ooc remote control system code "1", "0", "F" (floating)  */
  MENU_VAR_RC_DEV,              /* ooc remote control device code "A", "B", "C", "D", "E"   */
  MENU_VAR_RC_CMD,              /* ooc remote control device code "on", "off", "on_off", "on_F", "off_F"   */
  MENU_VAR_BOUNDARY_CNT,
} MENU_VAR_BOUNDARIES;

typedef struct
{
  uint8_t x;
  uint8_t y;
  uint8_t item_type;
} MENU_ITEM;

enum
{
  MENU_N_BLIND_HOUR_UP,
  MENU_N_BLIND_MINUTE_UP,
  MENU_N_BLIND_HOUR_DOWN,
  MENU_N_BLIND_MINUTE_DOWN,
  MENU_N_BLIND_VALUE_CNT
} MENU_N_BLIND_VALUE;

enum
{
  MENU_N_BLIND_WINDOW,
  MENU_N_BLIND_WINDOW_LIGHTBOW,
  MENU_N_BLIND_WINDOW_VALUE_CNT
} MENU_N_BLIND_WINDOW_VALUE;

#define n_blind_page_to_day_month__(_page, _day, _month)      \
{                                                             \
  _day   = (_page & 1) ? 15 : 1;                              \
  _month = (_page/2) + 1;                                     \
}

#define n_blind_day_month_to_page__(_day, _month, _page)      \
{                                                             \
  _page = (_month * 2) - 1;                                   \
  if(_day < 15) _page--;                                      \
}

/* 6.3. functions                                                          */
extern void     menu_init                   (void);
extern void     menu_srv                    (void);
extern void     menu_selection              (uint8_t menu_new);
extern uint8_t  menu_trim_var               (uint8_t index, uint8_t value);
extern void     menu_item_write             (uint8_t value, uint8_t item);
extern uint8_t  menu_srv_dec_inc_item       (uint8_t * current_item, uint8_t item_max);
extern uint8_t  menu_srv_dec_inc_value      (uint8_t item_type, uint8_t * value);
extern uint16_t menu_srv_dec_inc_value_u16  (uint16_t value);
extern void     menu_setup_item_stack       (uint8_t * PROGMEM src, uint8_t len);

extern void  menu_main_coldstart_init       (void);
extern void  menu_main_init                 (void);
extern void  menu_main_srv                  (void);

extern void     menu_n_blind_coldstart_init  (void);
extern void     menu_n_blind_init            (void);
extern void     menu_n_blind_srv             (void);

extern void  menu_d_blind_coldstart_init        (void);
extern void  menu_d_blind_init                  (void);
extern void  menu_d_blind_srv                   (void);

extern void  menu_rc_timer_coldstart_init       (void);
extern void  menu_rc_timer_init                 (void);
extern void  menu_rc_timer_srv                  (void);

extern void  menu_dt_coldstart_init   (void);
extern void  menu_dt_init             (void);
extern void  menu_dt_srv              (void);

extern void  menu_motor_coldstart_init       (void);
extern void  menu_motor_init                 (void);
extern void  menu_motor_srv                  (void);

extern void  menu_reboots_coldstart_init     (void);
extern void  menu_reboots_init               (void);
extern void  menu_reboots_srv                (void);

extern void  menu_misc_coldstart_init       (void);
extern void  menu_misc_init                 (void);
extern void  menu_misc_srv                  (void);

extern void  menu_adv_coldstart_init       (void);
extern void  menu_adv_init                 (void);
extern void  menu_adv_srv                  (void);

extern void  menu_curr_value_coldstart_init (void);
extern void  menu_curr_value_init           (void);
extern void  menu_curr_value_srv            (void);

/************************************************************************/
/* 7. alarm manager                                                     */
/************************************************************************/

/* 7.1. defines                                                         */
#define ALM_M_AUTO_MODE_OFF                         0
#define ALM_M_AUTO_MODE_INITIALIZED                 1
#define ALM_M_AUTO_MODE_DOWNDRIVEN                  2
#define ALM_M_AUTO_MODE_UPDRIVEN                    3
#define ALM_M_AUTO_MODE_DRIVING_TO_HIDDEN_SENSOR    4
#define ALM_M_AUTO_MODE_DRIVING_TO_UNHIDDEN_SENSOR  5


/* 7.2. types                                                      */

typedef struct
{
    uint8_t  sensor_upper_bound;                                /* the sensor value has reached the upper bound for a certain time */
    uint8_t  sensor_lower_bound;                                /* the sensor value has reached the lower bound for a certain time */

    uint8_t   state;                                            /* auto mode is state: ALM_M_AUTO_MODE_OFF, ALM_M_AUTO_MODE_ENABLED, ALM_M_AUTO_MODE_DOWN, ALM_M_AUTO_MODE_UP */
}ALM_SHADOWING_SM;                                              /* the alm_manager is controlling the day blind automatically */

/* 7.3. functions                                                       */

extern void  alm_manager_init                             (void);
extern void  alm_manager_perform_n_blind_request_if_due   (void);
extern void  alm_manager_check_request                    (void);
extern void  alm_manager_perform_d_blind_request_callback (void);

/************************************************************************/
/* 8. eeprom                                                            */
/************************************************************************/

/* 8.1. layout                                                            */

#define EEPROM_ADDRESS(_addr) (uint16_t) (&(((NV_DATA *) 0) -> _addr))

/* 8.2. functions                                                         */

extern void eeprom_read_var_from_eeprom   (uint8_t * dst_add, uint16_t src_add, uint16_t len);
extern void eeprom_write_var_to_eeprom    (uint16_t dst_add, uint8_t * src_add, uint16_t len);

/************************************************************************/
/* 9. bright sensors                                                    */
/************************************************************************/

/* 9.1. layout                                                            */

#define SENSOR_COUNT                      8
#define SENSOR_HIDDEN_BY_BLIND_VALUE     50 /* if a sensor is hidden by its blind it has at least or lower that value */

/* 9.2. functions                                                         */

void sensor_init                 (void);
void sensor_start_adc_conversion (void);
void sensor_check_adc_conversion (void);

/************************************************************************/
/* 10. motor                                                            */
/************************************************************************/

/* 10.1. defines                                                          */


#if defined MOTOR_MODE_DIR_PWR

#define MOTOR_STATE_SHIFT_DIR     1 /* bit position #1: direction */
#define MOTOR_STATE_SHIFT_PWR     0 /* bit position #0: power     */


#define MOTOR_OPC_HALT          ((0<<MOTOR_STATE_SHIFT_DIR) | (0<<MOTOR_STATE_SHIFT_PWR))
#define MOTOR_OPC_UP            ((0<<MOTOR_STATE_SHIFT_DIR) | (1<<MOTOR_STATE_SHIFT_PWR))
#define MOTOR_OPC_DOWN          ((1<<MOTOR_STATE_SHIFT_DIR) | (1<<MOTOR_STATE_SHIFT_PWR))

#else

#define MOTOR_STATE_SHIFT_DOWN     1 /* bit position #1: down */
#define MOTOR_STATE_SHIFT_UP       0 /* bit position #0: up   */


#define MOTOR_OPC_HALT          ((0<<MOTOR_STATE_SHIFT_UP) | (0<<MOTOR_STATE_SHIFT_DOWN))
#define MOTOR_OPC_UP            ((1<<MOTOR_STATE_SHIFT_UP) | (0<<MOTOR_STATE_SHIFT_DOWN))
#define MOTOR_OPC_DOWN          ((0<<MOTOR_STATE_SHIFT_UP) | (1<<MOTOR_STATE_SHIFT_DOWN))
#define MOTOR_OPC_PGM           ((1<<MOTOR_STATE_SHIFT_UP) | (1<<MOTOR_STATE_SHIFT_DOWN))

#endif

#define MOTOR_RUNTIME_FULL      0

/* 10.2. types                                                              */

typedef struct
{
  uint8_t       rl_number;
  uint8_t       opc;
  uint8_t       runtime;

}MOTOR_REQ;

/* 10.3. functions                                                          */

extern void  motor_init                 (void);
extern void  motor_update_driving_time  (void);
extern uint8_t  motor_request           (MOTOR_REQ *);
extern void  motor_check_new_request    (void);
extern void  motor_service_rc_telegram  (void);

/************************************************************************/
/* 11. wall buttons                                                     */
/************************************************************************/

/* 11.1. defines                                                          */

/* 11.2. functions                                                        */

extern void   wb_init                     (void);
extern void   wb_enter_halt_state         (uint8_t rl_count);
extern void   wb_evaluate                 (void);

/************************************************************************/
/* 12. blind setup service                                              */
/************************************************************************/

/* 12.1. defines                                                          */
#define SERVICE_MODE_INTERVALL_COUNT    3       /* button must be pressed for SERVICE_MODE_INTERVALL_COUNT */
#define SERVICE_MODE_INTERVALL_TIME     200     /* within SERVICE_MODE_INTERVALL_TIME (multiple of 10ms)   */

#define SERVICE_MODE_CODE_CHANGE_TIME        40     /* multiple of 100ms                                       */

#define SERVICE_MODE_EVAL_NOT_DONE            0
#define SERVICE_MODE_EVAL_DONE                1
#define SERVICE_MODE_EVAL_SYS_CODE_MISMATCH   2

#define SERVICE_MODE_OFF        0
#define SERVICE_MODE_N_BLIND    1
#define SERVICE_MODE_D_BLIND    2
#define SERVICE_MODE_MOTOR      3
#define SERVICE_MODE_GLOBAL     4
#define SERVICE_MODE_IDLE       5

/* 12.2. functions                                                        */

extern void     service_mode_init                      (void);
extern void     service_mode_entering_intervall_reset  (void);
extern void     service_mode_check_enter_exit          (uint8_t bank, uint8_t rl_number);
extern uint8_t  service_mode_evaluation                (void);

/************************************************************************/
/* 13. data                                                             */
/************************************************************************/


#define RL_COUNT                8  /* is fixed to #8 as every bit in a byte represents on motor resp. assigned switch */

#define RL_PROP_SUPPORT_N_BLIND   (uint8_t) 0x01
#define RL_PROP_SUPPORT_RC_RCVE   (uint8_t) 0x02
#define RL_PROP_SUPPORT_ADVENT    (uint8_t) 0x04

#pragma pack(1)
/* marshalling of data in eeprom (copied to ram) */
typedef struct
{
  uint8_t hour_up;
  uint8_t minute_up;
  uint8_t hour_down;
  uint8_t minute_down;
} NV_STRUC_N_BLIND;

typedef struct
{
  uint8_t hour_down;
  uint8_t minute_down;
  uint8_t hour_up;
  uint8_t minute_up;
  uint8_t sensor_down;
  uint8_t sensor_up;
  uint8_t time_hyst;
  uint8_t enable;
} NV_STRUC_D_BLIND;

typedef struct
{
  uint8_t runtime;                                     /* motor runtime */
  uint8_t rc_code[2];                                  /* remote control telegram assigned to the motor */
} NV_STRUC_MOTOR;

typedef struct
{
  uint8_t hour_on;
  uint8_t minute_on;
  uint8_t hour_off;
  uint8_t minute_off;
  uint8_t weekdays;
  uint8_t rc_code[2];                                  /* remote control telegram assigned to the timer */
} NV_STRUC_TIMER;

typedef struct
{
  uint8_t morning_hour_on;
  uint8_t morning_minute_on;
  uint8_t morning_hour_off;
  uint8_t morning_minute_off;
  uint8_t evening_hour_on;
  uint8_t evening_minute_on;
  uint8_t evening_hour_off;
  uint8_t evening_minute_off;
  uint8_t rc_code[2];                                  /* remote control telegram assigned to the timer */
  uint16_t has_lightbow;
} NV_STRUC_ADV;

typedef struct
{
  uint16_t included;
} NV_STRUC_N_BLIND_WINDOW;

typedef struct
{
  uint8_t properties;
} NV_STRUC_MISC;

#define RL_BOOT_ENTRIES   4     /* we have a 4 line display and keep 4 bootentries in eeprom */

typedef struct
{
  uint16_t reboot_ctr;
  uint8_t index;
  struct
  {
    uint8_t watch_el[4]; /* date/time entry contains day, month, hour, minute */
    uint8_t reset_flag;
    uint8_t reserved;
    uint16_t line;
  } entry[RL_BOOT_ENTRIES];
} NV_STRUC_BOOTTIME;

typedef struct
{
  NV_STRUC_N_BLIND        n_blind[RL_N_BLIND_TIME_PAGES];
  NV_STRUC_D_BLIND        d_blind[RL_COUNT];
  NV_STRUC_MOTOR          motor[RL_COUNT_FULL];
  NV_STRUC_TIMER          timer[RL_RC_TIMER_PAGES];
  NV_STRUC_ADV            adv;
  NV_STRUC_N_BLIND_WINDOW n_blind_window;
  NV_STRUC_MISC           misc;
  NV_STRUC_BOOTTIME       btime;
} NV_DATA;

NV_DATA nv_data;                     /* non volatile data copied from eeprom to ram */

#pragma pack()

typedef struct
{
  struct
  {
    uint8_t watch_el[RV3029_WATCH_EL_CNT];
    uint8_t dst_flag;               /* daytime saving time flag */
  } rv3029;

  struct
  {
    uint8_t current_second; /* free running seconds */
  } time;

  struct
  {
    uint8_t single[SENSOR_COUNT];    /* average value of every sensor */
    uint8_t index;
  } sensor;

  struct
  {
    uint8_t index;
    MENU_ITEM item_stack[MENU_ITEM_STACK_SIZE];   /* the item_stack holds the description of the items of the current menu page */
    struct
    {
      uint8_t current_page; /* current 15-days page */
      uint8_t current_rl;   /* associated room with blind */
      uint8_t current_item; /* editable item */
    }n_blind;
    struct
    {
      uint8_t current_rl;   /* associated room with blind */
      uint8_t current_item; /* editable item */
    }d_blind;
    struct
    {
      uint8_t current_rl;
      uint8_t current_item;
    }rt_motor;
    struct
    {
      uint8_t current_item;
      uint8_t current_page;
    }rc_timer;
    struct
    {
      uint8_t current_item;
      uint8_t watch_el[RV3029_WATCH_EL_CNT];
    }dt;
    struct
    {
      uint8_t current_item;
    }misc;
    struct
    {
      uint8_t current_rl;
      uint8_t current_item;
    }adv;
    struct
    {
      uint8_t trigger;
      uint8_t page;
      uint8_t max_item;
    }curr_value;
    struct
    {
      uint8_t start_index;
      uint8_t current_index;
    }main;
  } menu;

  struct
  {
    ALM_SHADOWING_SM shadowing_sm[RL_COUNT];

    uint8_t  current_page;                                         /* current 15-days page */
    uint8_t  trigger;
    uint8_t  timer_page_index;                                     /* global var as it also allows to resume checking timer events                                    */
    uint8_t  n_blind_req_opc;                                      /* motor opcode */
    uint8_t  n_blind_req_delay;
    uint16_t n_blind_req_bitlane;                                  /* bit coded field containing blinds those are to be driven up sequentially by night-blind request */
    uint16_t n_blind_tmp_exclude_bitlane;                          /* bit coded field containing blinds those are temporarily not under night-blind control           */
  }alm;

  struct
  {
    uint8_t perform_stop_req;
    uint8_t current_runtime[RL_COUNT];                  /* time in second while motor is running         */
    uint8_t service_runtime;                            /* time in second while motor is running in service mode        */
  }motor;

  struct
  {

    KEY_BANK bank[KEY_BANK_MAX];          /* KEY_BANK_MAX: one byte of the keypad side of MCP23017 (MCP23017_ADDRESS_HMI) and two bytes of the MCP23017 (MCP23017_ADDRESS_WB) */

    uint8_t wb_debounce_state;
    uint8_t kp_debounce_state;
    uint8_t wb_trigger_read_input;
    uint8_t kp_trigger_read_input;
    uint8_t evaluate;
    struct
    {
      uint8_t button;
      uint8_t entered;
      uint8_t time_accept_rc_code_change;
      uint8_t intervall_count;
      uint8_t intervall_time;
    }service_mode;
  }key;


  struct
  {
    uint8_t state[RL_COUNT];
  }wb;

  struct
  {
    uint8_t hmi_detect;
  }system;

  struct
  {
    uint8_t evaluate;             /* count of rceived bits */
    uint8_t timeout;
    uint8_t bit_location;         /* location of a received bit within a byte  */
    uint8_t byte_location;        /* location of a received bit within a byte  */
    uint8_t time_multiplier;      /* use a multiple of two tacts waiting for silence  */
    uint8_t last_tel_raw[3];      /* last valid received telegram stream       */
    uint8_t curr_tel_raw[3];      /* telegram about to be received             */
    struct
    {
      uint8_t raw[3];             /* telegram pushed to user                      */
      uint8_t device_code;           /* first device that was found in the telegram  */
      uint8_t on_off_code;        /* on off code that was found in the telegram   */
    }user_tel;
    uint8_t state;
    uint8_t key[KEY_BANK_MAX];    /* keypad not supported on remote control -> dummy */
  }rc_rx;

  struct
  {
    uint8_t telegram[3];          /* telegram to be issued                          */
    uint8_t repetition;           /* how many times is the telegram to be repeated  !=0 -> transmitter busy */
    uint8_t bit_location;         /* location of a received bit within a byte       */
    uint8_t byte_location;        /* location of a received bit within a byte       */
    uint8_t busy;                 /* reansmitter is busy sending a command          */
    uint8_t test_trigger;         /* reansmitter is busy sending a command          */

    struct
    {
      uint8_t rd_idx;
      uint8_t wr_idx;
      struct
      {
          RC_COMMAND rc_command;
          uint8_t    on_off_command;
          uint8_t    repetition_cnt;
      } req[RC_TX_QUEUE_SIZE];
    } queue;
  }rc_tx;
} RL_STRUCT_VAR;


extern  volatile RL_STRUCT_VAR   rl_data;

typedef struct
{
  uint16_t line;
} RL_STRUCT_NOINIT_VAR;

extern  volatile RL_STRUCT_NOINIT_VAR   rl_noinit_data;

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

