/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/***********************************************************************************/
/* rip off from atmel: the function writes a single byte to an eeprom address      */
/***********************************************************************************/
static void eeprom_write(unsigned int ucAddress, uint8_t ucData)
{
#if defined CPU_ATMEGA16
  /* Wait for completion of previous write */
  while(EECR & (1<<EEWE))
  ;
  /* Set up address and data registers */
  EEAR = ucAddress;
  EEDR = ucData;
  /* Write logical one to EEMWE */
  EECR |= (1<<EEMWE);
  /* Start eeprom write by setting EEWE */
  EECR |= (1<<EEWE);
#elif defined CPU_ATMEGA168
  /* Wait for completion of previous write */
  while(EECR & (1<<EEPE));

  /* Set Programming mode */
  EECR = (0<<EEPM1)|(0<<EEPM0);

  /* Set up address and data registers */
  EEAR = ucAddress;
  EEDR = ucData;

  /* Write logical one to EEMPE */
  EECR |= (1<<EEMPE);

  /* Start eeprom write by setting EEPE */
  EECR |= (1<<EEPE);
#endif
}

/***********************************************************************************/
/* rip off from atmel: the function reads a single byte from an eeprom address     */
/***********************************************************************************/
static uint8_t eeprom_read(unsigned int ucAddress)
{
#if defined CPU_ATMEGA16
  /* Wait for completion of previous write */
  while(EECR & (1<<EEWE))
  ;
  /* Set up address register */
  EEAR = ucAddress;
  /* Start eeprom read by writing EERE */
  EECR |= (1<<EERE);
  /* Return data from data register */
  return EEDR;
#elif defined CPU_ATMEGA168
  /* Wait for completion of previous write */
  while(EECR & (1<<EEPE));

  /* Set up address register */
  EEAR = ucAddress;

  /* Start eeprom read by writing EERE */
  EECR |= (1<<EERE);

  /* Return data from data register */
  return EEDR;
#endif
}


/* read eeprom into var field */
void eeprom_read_var_from_eeprom  (uint8_t * dst_add, uint16_t src_add, uint16_t len)
{
  while(len--)
  {
    * dst_add++ = eeprom_read(src_add++);
  }
}

/* write var field into eeprom */
void eeprom_write_var_to_eeprom  (uint16_t dst_add, uint8_t * src_add, uint16_t len)
{
  while(len--)
  {
    eeprom_write(dst_add++, * src_add++);
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

