/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"
#include <avr/wdt.h>

/* the human machine interface including a keypad with cursor cross and additional left and right button and a 4*20 character display */
#if defined CPU_ATMEGA16

  #define   HMI_DETECT         !( PINC & (1<<PINC4) )

#elif defined CPU_ATMEGA168p

  #define   HMI_DETECT         !( PIND & (1<<PIND5) )

#endif


/* initialise the hmi on startup if it has been detected */
void hmi_init (void)
{
  if( HMI_DETECT )
  {
    //  set addressing style
    mcp23017_write_register(MCP23017_ADDRESS_HMI+MCP23017_KP_BANK+MCP23017_IOCON, 0x80);

    _delay_ms(5);                  // delay for rearranging register addresses

    mcp23017_init_for_input(MCP23017_ADDRESS_HMI+MCP23017_KP_BANK);

    /* initialize display, cursor off */
    lcd_init(LCD_DISP_ON);

    rl_data.system.hmi_detect = 1;
  }
  else
  {
    rl_data.system.hmi_detect = 0;
  }
}

/* check the plug or play of the hmi regulary and perform a restart on change */
void hmi_pull_plug_check (void)
{
  if(   ( (HMI_DETECT) && (rl_data.system.hmi_detect == 0))
     || (!(HMI_DETECT) && (rl_data.system.hmi_detect == 1))
    )
  {
      for(;;);
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

