/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* elemental functions of the mcp23017 io-expander */
#define MCP23017_BASE_ADDRESS     0x40    /* fixed by chip - must not be changed */


/* write a byte to an output bench of mcp23017: output benches are the lcd-display of hmi the relay driver (motor PWR/UP, motor DIR/DOWN) */
void mcp23017_out(uint8_t address_bank, uint8_t data)
{
	/* write process value */
	mcp23017_write_register(address_bank+MCP23017_OLAT, data);
}

/* read a byte from an input bench of mcp23017: input benches are the keypad of hmi, the up wallbutton, the down wallbutton */
uint8_t mcp23017_in(uint8_t address_bank)
{
  uint8_t address = MCP23017_BASE_ADDRESS + ((address_bank>>4) & 0x0E);
  uint8_t reg     = (address_bank & 0x10) + MCP23017_GPIO;
  uint8_t data;

  /* read process value */
  i2c_start_wait(address + I2C_WRITE);
  i2c_write(reg);
  i2c_rep_start(address + I2C_READ);
  data = i2c_readNak();
  i2c_stop();

  return data;
}

/* write a dedicated register of the mcp23017: addressed by base offset, bench and register offset */
void mcp23017_write_register(uint8_t address_bank_reg, uint8_t data)
{
  uint8_t address = MCP23017_BASE_ADDRESS + ((address_bank_reg>>4) & 0x0E);
  uint8_t reg     = address_bank_reg & 0x1F;

	i2c_start_wait(address + I2C_WRITE);
	i2c_write(reg);
	i2c_write(data);
	i2c_stop();
}

/* this routine initialises a mcp23017 bench for input */
void mcp23017_init_for_input(uint8_t address_bank)
{
  /* set input direction */
  mcp23017_write_register(address_bank+MCP23017_IODIR, 0xFF);

  /* give a 5 volts reference to the input */
  mcp23017_out(address_bank, 0xFF);

  /* invert value */
  mcp23017_write_register(address_bank+MCP23017_IPOL, 0xFF);

  /* defval */
  mcp23017_write_register(address_bank+MCP23017_DEFVAL, 0x00);

  /* intcon */
  mcp23017_write_register(address_bank+MCP23017_INTCON, 0x00);

  /* gpinten */
  mcp23017_write_register(address_bank+MCP23017_GPINTEN, 0xFF);

  //mcp23017_in(address_bank);

  //  set addressing style
  //mcp23017_write_register(address_bank+MCP23017_IOCON, 0x80);

  /* set pullup for the input */
  mcp23017_write_register(address_bank+MCP23017_GPPU, 0xFF);
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

