/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating advent menu page */

const  PROGMEM char menu_adv_line_1[] = {"Fr\365h:  07:00 - 09:"}; /* Frueh */
const  PROGMEM char menu_adv_line_2[] = {"Sp\341t:  16:00 - 23:"}; /* Spaet */
const  PROGMEM char menu_adv_line_3[] = {"Code:"};

#define MENU_ITEM_ADV_MAX 16
const PROGMEM MENU_ITEM menu_item_adv[MENU_ITEM_ADV_MAX] =
                                        { {7, 0, MENU_VAR_HOUR}, {10, 0, MENU_VAR_MINUTE}, {15, 0, MENU_VAR_HOUR}, {18, 0, MENU_VAR_MINUTE},
                                          {7, 1, MENU_VAR_HOUR}, {10, 1, MENU_VAR_MINUTE}, {15, 1, MENU_VAR_HOUR}, {18, 1, MENU_VAR_MINUTE},
                                          {7, 2, MENU_VAR_RC_SYS}, {8, 2, MENU_VAR_RC_SYS}, {9, 2, MENU_VAR_RC_SYS}, {10, 2, MENU_VAR_RC_SYS}, {11, 2, MENU_VAR_RC_SYS}, {13, 2, MENU_VAR_RC_DEV},
                                          {0, 3, MENU_VAR_RL_CNT_FULL}, {15, 3, MENU_VAR_BOOLEAN}
                                        };

#if (MENU_ITEM_ADV_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif

/* coldstart initialisation of advent menu page */
void menu_adv_coldstart_init (void)
{
  rl_data.menu.adv.current_item =
  rl_data.menu.adv.current_rl   = 0;
}


/* init function of menu page "schwibbogen" - called once if menu page has changed */
void menu_adv_init (void)
{
  uint8_t item;

  lcd_clrscr();

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_adv[0], MENU_ITEM_ADV_MAX);
  lcd_puts_p(0, 0, &menu_adv_line_1[0]);
  lcd_puts_p(0, 1, &menu_adv_line_2[0]);
  lcd_puts_p(0, 2, &menu_adv_line_3[0]);

  item=0;

  for(item=0;item<8; item++)
  {
    menu_item_write( *(((uint8_t *) &nv_data.adv) + item), item );
  }

  uint8_t sys_code_digit_nr;
  uint8_t digit;
  /* write remote control system- and device code */
  for(sys_code_digit_nr=0; sys_code_digit_nr<5; sys_code_digit_nr++)
  {
    /* write 5 digit system code */
    digit = rc_get_syscode_digit_from_device ((uint8_t *) &nv_data.adv.rc_code[0], sys_code_digit_nr);
    menu_item_write(digit, item++);
  }

  /* write 1 digit device code */
  digit = rc_get_devcode_digit_from_device (nv_data.adv.rc_code[1]);
  menu_item_write(digit, item++);

  /* write room */
  menu_item_write(rl_data.menu.adv.current_rl, item++);

  /* write rooms yes/no decision */
  menu_item_write((nv_data.adv.has_lightbow & _BV(rl_data.menu.adv.current_rl)) ? 1 : 0, item++);

  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.adv.current_item].x, rl_data.menu.item_stack[rl_data.menu.adv.current_item].y);
  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "date/time setup" - called on keypad key change */
void menu_adv_srv (void)
{
  uint8_t current_item = rl_data.menu.adv.current_item;
  uint8_t item_type = rl_data.menu.item_stack[current_item].item_type;

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
      case KP_CURSOR_LEFT:
      case KP_CURSOR_RIGHT:
      {
          current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.adv.current_item, MENU_ITEM_ADV_MAX);

          lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

          break;
      }

      case KP_CURSOR_UP:
      case KP_CURSOR_DOWN:
      {

        uint8_t value;

        if( current_item <8 ) /* MENU_VAR_HOUR, MENU_VAR_MINUTE: select switch on /switch off time */
        {
          value = menu_srv_dec_inc_value(item_type, ((uint8_t *) &nv_data.adv)+current_item);

          /* write the desired item to display */
          menu_item_write(value, current_item);
        }
        else if( (current_item >=8) && (current_item <=12) ) /* MENU_VAR_RC_SYS select system code digit */
        {
          value = rc_get_syscode_digit_from_device ((uint8_t *) &nv_data.adv.rc_code[0], current_item-8);
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) & value);
          rc_set_syscode_digit_to_device ((uint8_t *) &nv_data.adv.rc_code[0], current_item-8, value);
          menu_item_write(value, current_item);
        }
        else if(current_item == 13) /* MENU_VAR_RC_DEV select device code digit */
        {
          uint8_t value;

          value = rc_get_devcode_digit_from_device (nv_data.adv.rc_code[1]);
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) & value);
          rc_set_devcode_digit_to_device ( (uint8_t *) &nv_data.adv.rc_code[1], value);
          menu_item_write(value, current_item);
        }
        else if(current_item == 14) /* room */
        {
          menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.adv.current_rl);

          /* write room */
          menu_item_write(rl_data.menu.adv.current_rl, 14);

          /* write rooms yes/no decision */
          menu_item_write((nv_data.adv.has_lightbow & _BV(rl_data.menu.adv.current_rl)) ? 1 : 0, 15);

          /* set cursor to the last edited value of menu and switch on cursor */
          lcd_gotoxy(rl_data.menu.item_stack[14].x, rl_data.menu.item_stack[14].y);
        }
        else if(current_item == 15) /* room yes/no decision */
        {
          nv_data.adv.has_lightbow ^= _BV(rl_data.menu.adv.current_rl);

          menu_item_write((nv_data.adv.has_lightbow & _BV(rl_data.menu.adv.current_rl)) ? 1 : 0, 15);
        }

        break;
      }

      case KP_CURSOR_MIDDLE:
      {
          eeprom_write_var_to_eeprom(EEPROM_ADDRESS(adv), (uint8_t *) &nv_data.adv, sizeof(NV_STRUC_ADV));

          /* leave this menu (escape) */
          //menu_selection(MENU_INDEX_MAIN);

          break;
      }

      case KP_RIGHT:
      {
        eeprom_read_var_from_eeprom((uint8_t *) &nv_data.adv, EEPROM_ADDRESS(adv), sizeof(NV_STRUC_ADV));

        /* leave this menu (escape) */
        menu_selection(MENU_INDEX_MAIN);

        break;
      }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

