/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"


/* this menu shows the current values of date, time, bright sensors and rtc chip temperature */
const  PROGMEM char menu_curr_value_page_0_line_1[] = "--.--.20-- --:--:--";

const  PROGMEM char menu_curr_value_page_1_line_1[] = "S1: - --- S2:";
const  PROGMEM char menu_curr_value_page_1_line_2[] = "S3: - --- S4:";
const  PROGMEM char menu_curr_value_page_1_line_3[] = "S5: - --- S6:";
const  PROGMEM char menu_curr_value_page_1_line_4[] = "S7: - --- S8:";

#define MENU_ITEM_CURR_VALUE_PAGE_0_MAX 14
const PROGMEM MENU_ITEM menu_item_curr_value_page_0[MENU_ITEM_CURR_VALUE_PAGE_0_MAX] =
                                                      { {0, 0, MENU_VAR_DAY}, {3, 0, MENU_VAR_MONTH}, {8, 0, MENU_VAR_YEAR}, {11, 0, MENU_VAR_HOUR}, {14, 0, MENU_VAR_MINUTE}, {17, 0, MENU_VAR_SECOND},
                                                        {0, 1, MENU_VAR_TEMPERATURE},
                                                        {0, 2, MENU_VAR_RC_SYS}, {1, 2, MENU_VAR_RC_SYS}, {2, 2, MENU_VAR_RC_SYS}, {3, 2, MENU_VAR_RC_SYS}, {4, 2, MENU_VAR_RC_SYS}, {6, 2, MENU_VAR_RC_DEV}, {8, 2, MENU_VAR_RC_CMD},
                                                      };

#if (MENU_ITEM_CURR_VALUE_PAGE_0_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif


#define MENU_ITEM_CURR_VALUE_PAGE_1_MAX 16
const PROGMEM MENU_ITEM menu_item_curr_value_page_1[MENU_ITEM_CURR_VALUE_PAGE_1_MAX] =
                                                      { {6, 0, MENU_VAR_SENSOR}, {16, 0, MENU_VAR_SENSOR},
                                                        {6, 1, MENU_VAR_SENSOR}, {16, 1, MENU_VAR_SENSOR},
                                                        {6, 2, MENU_VAR_SENSOR}, {16, 2, MENU_VAR_SENSOR},
                                                        {6, 3, MENU_VAR_SENSOR}, {16, 3, MENU_VAR_SENSOR},
                                                        {4, 0, MENU_VAR_SHADOWING_SM}, {14, 0, MENU_VAR_SHADOWING_SM},
                                                        {4, 1, MENU_VAR_SHADOWING_SM}, {14, 1, MENU_VAR_SHADOWING_SM},
                                                        {4, 2, MENU_VAR_SHADOWING_SM}, {14, 2, MENU_VAR_SHADOWING_SM},
                                                        {4, 3, MENU_VAR_SHADOWING_SM}, {14, 3, MENU_VAR_SHADOWING_SM},
                                                      };

#if (MENU_ITEM_CURR_VALUE_PAGE_1_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif

/* coldstart initialisation of menu page "current values" */
void menu_curr_value_coldstart_init (void)
{
  rl_data.menu.curr_value.page = 0;
}

/* init function of menu page "current values" - called once if menu page has changed */
void menu_curr_value_init (void)
{
  lcd_clrscr();
  if(rl_data.menu.curr_value.page == 0)
  {
    rl_data.menu.curr_value.max_item = MENU_ITEM_CURR_VALUE_PAGE_0_MAX;
    menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_curr_value_page_0[0], MENU_ITEM_CURR_VALUE_PAGE_0_MAX);
    lcd_puts_p(0, 0, &menu_curr_value_page_0_line_1[0]);
  }
  else
  {
    rl_data.menu.curr_value.max_item = MENU_ITEM_CURR_VALUE_PAGE_1_MAX;
    menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_curr_value_page_1[0], MENU_ITEM_CURR_VALUE_PAGE_1_MAX);
    lcd_puts_p(0, 0, &menu_curr_value_page_1_line_1[0]);
    lcd_puts_p(0, 1, &menu_curr_value_page_1_line_2[0]);
    lcd_puts_p(0, 2, &menu_curr_value_page_1_line_3[0]);
    lcd_puts_p(0, 3, &menu_curr_value_page_1_line_4[0]);
  }
}


/* regular service function of menu page "current values" - called every second */
void menu_curr_value_srv (void)
{
  uint8_t current_item, item_type;

  for(current_item=0; current_item<rl_data.menu.curr_value.max_item; current_item++)
  {
      item_type = rl_data.menu.item_stack[current_item].item_type;

      if(item_type == MENU_VAR_SECOND )
      {
          menu_item_write(rl_data.time.current_second, current_item);
      }
      else if(item_type == MENU_VAR_TEMPERATURE )
      {
          #if 0
          rv3029_write_temperature(current_item);
          #endif
      }
      else if(item_type == MENU_VAR_SENSOR )
      {
        menu_item_write(rl_data.sensor.single[current_item], current_item);
      }
      else if(item_type == MENU_VAR_SHADOWING_SM )
      {
        menu_item_write(rl_data.alm.shadowing_sm[current_item-8].state, current_item);
      }
      else if(item_type == MENU_VAR_RC_SYS )
      {
        uint8_t value;

        if(rl_data.rc_rx.user_tel.on_off_code != RC_DIGIT_VALUE_NO)
        {
            value = rc_get_syscode_digit_from_device ((uint8_t *) &rl_data.rc_rx.user_tel.raw[0], current_item-7);
            menu_item_write(value, current_item);
        }
      }
      else if(item_type == MENU_VAR_RC_DEV )
      {
        if(rl_data.rc_rx.user_tel.on_off_code != RC_DIGIT_VALUE_NO)
        {
            menu_item_write(rl_data.rc_rx.user_tel.device_code, current_item);
        }
      }
      else if(item_type == MENU_VAR_RC_CMD )
      {
        if(rl_data.rc_rx.user_tel.on_off_code != RC_DIGIT_VALUE_NO)
        {
            menu_item_write(rl_data.rc_rx.user_tel.on_off_code, current_item);
        }
      }
      else /* MENU_VAR_DAY, MENU_VAR_MONTH, MENU_VAR_YEAR, MENU_VAR_HOUR, MENU_VAR_MINUTE */
      {
        menu_item_write(rl_data.rv3029.watch_el[item_type], current_item);
      }
  }

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
    case KP_CURSOR_UP:
    case KP_CURSOR_DOWN:
    {
      if(rl_data.menu.curr_value.page == 0)
      {
        rl_data.menu.curr_value.page = 1;
      }
      else
      {
        rl_data.menu.curr_value.page = 0;
      }

      menu_curr_value_init();

      break;
    }
    case KP_RIGHT:
    {
      /* leave this menu (escape) */
      menu_selection(MENU_INDEX_MAIN);

      break;
    }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

