/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating "day blind" menu page */

const  PROGMEM char menu_d_blind_line_1[] = "Ab/Auf:  12:00 14:";
const  PROGMEM char menu_d_blind_line_2[] = "Sensor:";
const  PROGMEM char menu_d_blind_line_3[] = "t-Hyst:  15min";

#define MENU_ITEM_D_BLIND_MAX 9
const PROGMEM MENU_ITEM menu_item_d_blind[MENU_ITEM_D_BLIND_MAX] =
                                            { {9, 0, MENU_VAR_HOUR}, {12, 0, MENU_VAR_MINUTE}, {15, 0, MENU_VAR_HOUR}, {18, 0, MENU_VAR_MINUTE},
                                              {9, 1, MENU_VAR_SENSOR}, {15, 1, MENU_VAR_SENSOR},
                                              {9, 2, MENU_VAR_MINUTE},
                                              {0, 3, MENU_VAR_RL_CNT}, {15, 3, MENU_VAR_BOOLEAN},
                                            };

#if (MENU_ITEM_D_BLIND_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif


/* coldstart initialisation of menu page "day blind" */
void menu_d_blind_coldstart_init  (void)
{
  /* start with the first rl */
  rl_data.menu.d_blind.current_rl=0;
  /* put the cursor to MENU_ITEM_N_BLIND_FIRST_EDITABLE (hour_up) */
  rl_data.menu.d_blind.current_item=7;
}


/* init function of menu page "day blind" - called once if menu page has changed */
void menu_d_blind_init	(void)
{
  lcd_clrscr();

  NV_STRUC_D_BLIND * current_d_blind = (NV_STRUC_D_BLIND *) &nv_data.d_blind[rl_data.menu.d_blind.current_rl];

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_d_blind[0], MENU_ITEM_D_BLIND_MAX);

  /* rewrite the last line showing room and its yes/no (character lines have different lengths) */
  /* write room */
  menu_item_write(rl_data.menu.d_blind.current_rl, 7);

  /* write rooms yes/no decision */
  menu_item_write(current_d_blind->enable, 8);

  if(current_d_blind->enable != 0)
  {
    lcd_puts_p(0, 0,  &menu_d_blind_line_1[0]);
    lcd_puts_p(0, 1,  &menu_d_blind_line_2[0]);
    lcd_puts_p(0, 2,  &menu_d_blind_line_3[0]);

    menu_item_write(current_d_blind->hour_down, 0);
    menu_item_write(current_d_blind->minute_down, 1);
    menu_item_write(current_d_blind->hour_up, 2);
    menu_item_write(current_d_blind->minute_up, 3);
    menu_item_write(current_d_blind->sensor_down, 4);
    menu_item_write(current_d_blind->sensor_up, 5);
    menu_item_write(current_d_blind->time_hyst, 6);
  }

  /* set cursor to the last edited value of alarm menu and switch on cursor */
  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.d_blind.current_item].x, rl_data.menu.item_stack[rl_data.menu.d_blind.current_item].y);

  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "day blind" - called on keypad key change */
void menu_d_blind_srv (void)
{
  NV_STRUC_D_BLIND * current_d_blind = (NV_STRUC_D_BLIND *) &nv_data.d_blind[rl_data.menu.d_blind.current_rl];

  uint8_t current_item = rl_data.menu.d_blind.current_item;
  //uint8_t item_type    = rl_data.menu.item_stack[current_item].item_type;

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
    case KP_CURSOR_LEFT:
    case KP_CURSOR_RIGHT:
    {
        if(current_d_blind->enable != 0)
        {
          current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.d_blind.current_item, MENU_ITEM_D_BLIND_MAX);
        }
        else
        {
          /* only changing the room is allowed */
          current_item = rl_data.menu.d_blind.current_item = (rl_data.menu.d_blind.current_item == 7) ? 8 : 7;
        }

        lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

        break;
    }

    case KP_CURSOR_UP:
    case KP_CURSOR_DOWN:
    {
        uint8_t item_type = rl_data.menu.item_stack[current_item].item_type;
        uint8_t value;

        if( current_item < 7 )
        {
          uint8_t * tmp_ptr = (uint8_t *) current_d_blind + current_item;
          value = menu_srv_dec_inc_value(item_type, tmp_ptr);
          menu_item_write(value, current_item);
        }
        else if(current_item == 7)       /* room selected */
        {
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.d_blind.current_rl);

          menu_d_blind_init();
        }
        else if(current_item == 8)  /* room yes/no selected */
        {
          value = menu_srv_dec_inc_value(item_type, (uint8_t *) & current_d_blind->enable);

          menu_d_blind_init();
        }

        break;
    }

    case KP_CURSOR_MIDDLE:
    {
        /* save var and leave menu */
        eeprom_write_var_to_eeprom(EEPROM_ADDRESS(d_blind[0]), (uint8_t *) &nv_data.d_blind[0], RL_COUNT * sizeof(NV_STRUC_D_BLIND));

        //menu_selection(MENU_INDEX_MAIN);

        break;
    }

    case KP_RIGHT:
    {
      /* reload var from eeprom and leave menu */
      eeprom_read_var_from_eeprom((uint8_t *) &nv_data.d_blind[0], EEPROM_ADDRESS(d_blind[0]), RL_COUNT * sizeof(NV_STRUC_D_BLIND));

      menu_selection(MENU_INDEX_MAIN);

      break;
    }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

