/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating "date/time setup" menu page */

const  PROGMEM char menu_dt_line_1[] = "Tag :   ,   .  .20";
const  PROGMEM char menu_dt_line_2[] = "Zeit:   :  :   Uhr";

#define MENU_ITEM_DT_MAX 7
const PROGMEM MENU_ITEM menu_item_dt[MENU_ITEM_DT_MAX] =
                                      { {6, 0, MENU_VAR_WEEKDAY}, {10, 0, MENU_VAR_DAY}, {13, 0, MENU_VAR_MONTH}, {18, 0, MENU_VAR_YEAR},
                                        {6, 1, MENU_VAR_HOUR}, {9, 1, MENU_VAR_MINUTE}, {12, 1, MENU_VAR_SECOND}
                                      };

#if (MENU_ITEM_DT_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif


/* coldstart initialisation of menu page "date/time setup" */
void menu_dt_coldstart_init (void)
{
  /* put the cursor to the first watch element to edit (weekday) */
  rl_data.menu.dt.current_item=0;
}


/* init function of menu page "date/time setup" - called once if menu page has changed */
void menu_dt_init (void)
{
  uint8_t menu_item;

  lcd_clrscr();

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_dt[0], MENU_ITEM_DT_MAX);

  lcd_puts_p(0, 0, &menu_dt_line_1[0]);
  lcd_puts_p(0, 1, &menu_dt_line_2[0]);

  rv3029_read_time();

  for(menu_item=0;menu_item<MENU_ITEM_DT_MAX; menu_item++)
  {
      /* we must use an intermediate field because from displaying date/time until pressing "set date/time" button the rl_data.rv3029.watch_el field might have been reread */
      /* trim the value that has been increased/decreased before to the boundary if needed */
      rl_data.menu.dt.watch_el[menu_item] = menu_trim_var (menu_item, rl_data.rv3029.watch_el[menu_item]);

      menu_item_write(rl_data.menu.dt.watch_el[menu_item], menu_item);
  }

  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.dt.current_item].x, rl_data.menu.item_stack[rl_data.menu.dt.current_item].y);
  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "date/time setup" - called on keypad key change */
void menu_dt_srv (void)
{
  uint8_t current_item = rl_data.menu.dt.current_item;
  uint8_t item_type    = rl_data.menu.item_stack[current_item].item_type;

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
      case KP_CURSOR_LEFT:
      case KP_CURSOR_RIGHT:
      {
          current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.dt.current_item, MENU_ITEM_DT_MAX);

          lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

          break;
      }

      case KP_CURSOR_UP:
      case KP_CURSOR_DOWN:
      {
          menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.dt.watch_el[current_item]);

          menu_item_write(rl_data.menu.dt.watch_el[current_item], current_item);

          break;
      }

      case KP_CURSOR_MIDDLE:
      {
        /* save var and leave menu */
          rv3029_set_time((uint8_t *) & rl_data.menu.dt.watch_el[0]);

          /* leave this menu (escape) */
          menu_selection(MENU_INDEX_MAIN);

          break;
      }

      case KP_RIGHT:
      {
        /* reload var from rv3029 and leave menu */
        rv3029_read_time();

        /* leave this menu (escape) */
        menu_selection(MENU_INDEX_MAIN);

        break;
      }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

