/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "string.h"
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating "menu main" menu page */


#define MENU_MAIN_1_STRING       "Nachtverdunklung"
#define MENU_MAIN_2_STRING       "Tagesabschattung"
#define MENU_MAIN_3_STRING       "Motor Setup"
#define MENU_MAIN_4_STRING       "Schwibbogen"
#define MENU_MAIN_5_STRING       "Zeitschaltuhr"
#define MENU_MAIN_6_STRING       "Datum/Zeit setzen"
#define MENU_MAIN_7_STRING       "Reboots ("
#define MENU_MAIN_8_STRING       "Verschiedenes"
#define MENU_MAIN_9_STRING       "aktuelle Werte"
#define MENU_MAIN_x_STRING_SIZE  20 /* main menu lines must not be longer then printable 19 characters (+ end of line delimiter) -- the first printable character is the "fat arrow right"  */

const  PROGMEM char menu_main_name[MENU_INDEX_MAX-1][MENU_MAIN_x_STRING_SIZE]  = {MENU_MAIN_1_STRING, MENU_MAIN_2_STRING, MENU_MAIN_3_STRING, MENU_MAIN_4_STRING, MENU_MAIN_5_STRING, MENU_MAIN_6_STRING, MENU_MAIN_7_STRING, MENU_MAIN_8_STRING, MENU_MAIN_9_STRING};


/* coldstart initialisation of menu page "menu main" */
void menu_main_coldstart_init (void)
{
  rl_data.menu.main.start_index   =
  rl_data.menu.main.current_index = 0;
}


/* init function of menu page "menu main" - called once if menu page has changed */
void menu_main_init	(void)
{
  uint8_t i;
  lcd_clrscr();

  /* switch off the cursor */
  lcd_command(LCD_DISP_ON);

  for(i=0; i<4; i++)
  {
    if(rl_data.menu.main.start_index+i == MENU_INDEX_REBOOTS)
    {
        /* quick and dirty as menu_main_name must be in PROGMEM */
        char buffer[MENU_MAIN_x_STRING_SIZE];
        buffer[0] = 0;
        strcat(&buffer[0], MENU_MAIN_7_STRING);
        itoa(nv_data.btime.reboot_ctr, &buffer[ strlen(MENU_MAIN_7_STRING) ], 10);
        strcat(&buffer[0], ")");
        lcd_puts(1, i, &buffer[0]);
    }
    else
    {
        lcd_puts_p(1, i, &menu_main_name[rl_data.menu.main.start_index+i][0]);
    }
  }

  lcd_putc(0, (rl_data.menu.main.current_index - rl_data.menu.main.start_index), LCD_CHAR_FAT_ARROW_RIGHT);
}


/* regular service function of menu page "menu main" - called on keypad key change */
void menu_main_srv (void)
{
  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
    case KP_CURSOR_UP:
    case KP_CURSOR_DOWN:
    {
      lcd_puts(0, (rl_data.menu.main.current_index - rl_data.menu.main.start_index), " ");

      if(rl_data.key.bank[KP_INDEX].pressed == KP_CURSOR_UP)
      {
        if(--rl_data.menu.main.current_index == 255)
        {
          rl_data.menu.main.current_index = MENU_INDEX_CURR_VALUE;
        }
      }
      else
      {
        if(++rl_data.menu.main.current_index == MENU_INDEX_CURR_VALUE+1)
        {
          rl_data.menu.main.current_index = 0;
        }
      }

      if (rl_data.menu.main.current_index < rl_data.menu.main.start_index)
      {
        rl_data.menu.main.start_index = rl_data.menu.main.current_index;

        menu_main_init();
      }
      else if ((rl_data.menu.main.current_index - rl_data.menu.main.start_index) > 3 )
      {
        rl_data.menu.main.start_index = rl_data.menu.main.current_index - 3;

        menu_main_init();
      }

      lcd_putc(0, (rl_data.menu.main.current_index - rl_data.menu.main.start_index), LCD_CHAR_FAT_ARROW_RIGHT);

      break;
    }
    case KP_CURSOR_MIDDLE:
    {
      menu_selection(rl_data.menu.main.current_index);

      break;
    }
    case KP_RIGHT:
    {
      rl_data.menu.index = MENU_INDEX_MAX;

      lcd_backlight_off();

      lcd_clrscr();

      break;
    }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

