/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* menu treating miscelleaneous stuff menu page */

const  PROGMEM char menu_misc_line_1[] = "Nachtverdunklung:";
const  PROGMEM char menu_misc_line_2[] = "Fernbedienung   :";
const  PROGMEM char menu_misc_line_3[] = "Adventmodus     :";

#define MENU_ITEM_MISC_MAX 3
const PROGMEM MENU_ITEM menu_item_misc[MENU_ITEM_MISC_MAX] =
                                        { {19, 0, MENU_VAR_BOOLEAN_02}, {19, 1, MENU_VAR_BOOLEAN_02}, {19, 2, MENU_VAR_BOOLEAN_02}
                                        };

#if (MENU_ITEM_MISC_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif


/* coldstart initialisation of menu page "date/time setup" */
void menu_misc_coldstart_init (void)
{
  /* put the cursor to the first watch element to edit (weekday) */
  rl_data.menu.misc.current_item=0;
}


/* init function of menu page "miscellaneous" - called once if menu page has changed */
void menu_misc_init (void)
{
  lcd_clrscr();

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_misc[0], MENU_ITEM_MISC_MAX);

  lcd_puts_p(0, 0, &menu_misc_line_1[0]);
  lcd_puts_p(0, 1, &menu_misc_line_2[0]);
  lcd_puts_p(0, 2, &menu_misc_line_3[0]);

  menu_item_write( (nv_data.misc.properties & RL_PROP_SUPPORT_N_BLIND), 0);
  menu_item_write( (nv_data.misc.properties & RL_PROP_SUPPORT_RC_RCVE), 1);
  menu_item_write( (nv_data.misc.properties & RL_PROP_SUPPORT_ADVENT),  2);

  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.misc.current_item].x, rl_data.menu.item_stack[rl_data.menu.misc.current_item].y);
  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "date/time setup" - called on keypad key change */
void menu_misc_srv (void)
{
  uint8_t current_item = rl_data.menu.misc.current_item;
  //uint8_t item_type    = rl_data.menu.item_stack[current_item].item_type;

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
      case KP_CURSOR_LEFT:
      case KP_CURSOR_RIGHT:
      {
          current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.misc.current_item, MENU_ITEM_MISC_MAX);

          lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

          break;
      }

      case KP_CURSOR_UP:
      case KP_CURSOR_DOWN:
      {
          uint8_t value = (nv_data.misc.properties ^= _BV(current_item));

          menu_item_write(value & _BV(current_item), current_item);

          break;
      }

      case KP_CURSOR_MIDDLE:
      {
          eeprom_write_var_to_eeprom(EEPROM_ADDRESS(misc), (uint8_t *) &nv_data.misc, sizeof(NV_STRUC_MISC));

          /* leave this menu (escape) */
          //menu_selection(MENU_INDEX_MAIN);

          break;
      }

      case KP_RIGHT:
      {
        eeprom_read_var_from_eeprom((uint8_t *) &nv_data.misc, EEPROM_ADDRESS(misc), sizeof(NV_STRUC_MISC));

        /* leave this menu (escape) */
        menu_selection(MENU_INDEX_MAIN);

        break;
      }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

