/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* treating "remote control timer" menu page */

const  PROGMEM char menu_rc_timer_line_1[]      = "So Mo Di Mi Do Fr Sa";
const  PROGMEM char menu_rc_timer_line_3[]      = "Zeit:   :   -   :";
const  PROGMEM char menu_rc_timer_line_4[]      = "Code:";

#define MENU_ITEM_RC_TIMER_MAX 19
const PROGMEM MENU_ITEM menu_item_rc_timer[MENU_ITEM_RC_TIMER_MAX] =
                                             { {1, 1, MENU_VAR_BOOLEAN_02}, {4, 1, MENU_VAR_BOOLEAN_02}, {7, 1, MENU_VAR_BOOLEAN_02}, {10, 1, MENU_VAR_BOOLEAN_02}, {13, 1, MENU_VAR_BOOLEAN_02},
                                               {16, 1, MENU_VAR_BOOLEAN_02}, {19, 1, MENU_VAR_BOOLEAN_02},
                                               {6, 2, MENU_VAR_HOUR}, {9, 2, MENU_VAR_MINUTE}, {14, 2, MENU_VAR_HOUR}, {17, 2, MENU_VAR_MINUTE},
                                               {6, 3, MENU_VAR_RC_SYS}, {7, 3, MENU_VAR_RC_SYS}, {8, 3, MENU_VAR_RC_SYS}, {9, 3, MENU_VAR_RC_SYS}, {10, 3, MENU_VAR_RC_SYS}, {12, 3, MENU_VAR_RC_DEV},
                                               {16, 3, MENU_VAR_BOOLEAN}, {19, 3, MENU_VAR_TIMER_PAGE}
                                             };

#if (MENU_ITEM_RC_TIMER_MAX > MENU_ITEM_STACK_SIZE)
  #error "item stack too small"
#endif



/* coldstart initialisation of menu page "rc_timer" */
void menu_rc_timer_coldstart_init  (void)
{
  rl_data.menu.rc_timer.current_item = 0;
  rl_data.menu.rc_timer.current_page = 0;
}

/* init function of menu page "rc_timer" - called once if menu page has changed */
void menu_rc_timer_write_editable_items (void)
{
  uint8_t i;

  uint8_t page = rl_data.menu.rc_timer.current_page;

  /* write editable parameters (weekday yes/no) */
  for(i=0; i<7; i++)
  {
    menu_item_write( (nv_data.timer[page].weekdays & _BV(i)), i);
  }

  /* write editable parameters (on/off alarm times) */
  menu_item_write(nv_data.timer[page].hour_on, 7);
  menu_item_write(nv_data.timer[page].minute_on, 8);
  menu_item_write(nv_data.timer[page].hour_off, 9);
  menu_item_write(nv_data.timer[page].minute_off, 10);

  uint8_t sys_code_digit_nr;
  uint8_t digit;
  /* write remote control system- and device code */
  for(sys_code_digit_nr=0; sys_code_digit_nr<5; sys_code_digit_nr++)
  {
    /* write 5 digit system code */
    digit = rc_get_syscode_digit_from_device ((uint8_t *) &nv_data.timer[page].rc_code[0], sys_code_digit_nr);
    menu_item_write(digit, 11+sys_code_digit_nr);
  }

  /* write 1 digit device code */
  digit = rc_get_devcode_digit_from_device (nv_data.timer[page].rc_code[1]);
  menu_item_write(digit, 16);

  lcd_putc(16, 3, LCD_CHAR_FAT_ARROW_UP_DOWN);

  menu_item_write(rl_data.menu.rc_timer.current_page, 18);

  /* set cursor to the last edited value of alarm menu and switch on cursor */
  lcd_gotoxy(rl_data.menu.item_stack[rl_data.menu.rc_timer.current_item].x, rl_data.menu.item_stack[rl_data.menu.rc_timer.current_item].y);
}

/* init function of menu page "rc_timer" - called once if menu page has changed */
void menu_rc_timer_init	(void)
{
  lcd_clrscr();

  menu_setup_item_stack( (uint8_t * PROGMEM) &menu_item_rc_timer[0], MENU_ITEM_RC_TIMER_MAX);

  /* write 1st line of menu */
  lcd_puts_p(0, 0,  &menu_rc_timer_line_1[0]);

  /* write 3rd line of menu */
  lcd_puts_p(0, 2,  &menu_rc_timer_line_3[0]);

  /* write 4th line of menu */
  lcd_puts_p(0, 3,  &menu_rc_timer_line_4[0]);

  menu_rc_timer_write_editable_items();

  lcd_command(LCD_DISP_ON_CURSOR);
}


/* regular service function of menu page "night blind" - called on keypad key change */
void menu_rc_timer_srv (void)
{
  uint8_t current_item = rl_data.menu.rc_timer.current_item;
  uint8_t item_type    = rl_data.menu.item_stack[current_item].item_type;

  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
    case KP_CURSOR_LEFT:
    case KP_CURSOR_RIGHT:
    {
        current_item = menu_srv_dec_inc_item ((uint8_t *) & rl_data.menu.rc_timer.current_item, MENU_ITEM_RC_TIMER_MAX);

        lcd_gotoxy(rl_data.menu.item_stack[current_item].x, rl_data.menu.item_stack[current_item].y);

        break;
    }

    case KP_CURSOR_UP:
    case KP_CURSOR_DOWN:
    {
      NV_STRUC_TIMER * timer_page = (NV_STRUC_TIMER *) & nv_data.timer[rl_data.menu.rc_timer.current_page];

      uint8_t value;

      if(current_item <=6) /* MENU_VAR_BOOLEAN_02: check/uncheck weekeday */
      {
        /* toggle inclusion of a weekday */
        value = (timer_page->weekdays ^= _BV(current_item));

        menu_item_write(value & _BV(current_item), current_item);
      }
      else if( (current_item >=7) && (current_item <=10) ) /* MENU_VAR_HOUR, MENU_VAR_MINUTE: select switch on /switch off time */
      {
        value = menu_srv_dec_inc_value(item_type, (uint8_t *) &(timer_page->hour_on)+current_item-7); /* tricky: item #7 is MENU_VAR_HOUR but var it is at index 0 -> subtract #7 */

        /* write the desired item to display */
        menu_item_write(value, current_item);
      }
      else if( (current_item >=11) && (current_item <=15) ) /* MENU_VAR_RC_SYS select system code digit */
      {
        value = rc_get_syscode_digit_from_device ((uint8_t *) &timer_page->rc_code[0], current_item-11);
        value = menu_srv_dec_inc_value(item_type, (uint8_t *) & value);
        rc_set_syscode_digit_to_device ((uint8_t *) &timer_page->rc_code[0], current_item-11, value);
        menu_item_write(value, current_item);
      }
      else if(current_item == 16) /* MENU_VAR_RC_DEV select device code digit */
      {
        uint8_t value;

        value = rc_get_devcode_digit_from_device (timer_page->rc_code[1]);
        value = menu_srv_dec_inc_value(item_type, (uint8_t *) & value);
        rc_set_devcode_digit_to_device ( (uint8_t *) &timer_page->rc_code[1], value);
        menu_item_write(value, current_item);
      }
      else if(current_item == 17) /* MENU_VAR_BOOLEAN manual switch off switch on the radio controlled outlet */
      {
        uint8_t  on_off;
        if(rl_data.key.bank[KP_INDEX].pressed == KP_CURSOR_UP)
        {
          on_off = RC_DIGIT_VALUE_ON;
        }
        else
        {
          on_off = RC_DIGIT_VALUE_OFF;
        }

        rc_tx_send_command((RC_COMMAND *) &timer_page->rc_code[0], on_off, RC_TX_DEFAULT_REPETITION_CNT);
      }
      else if(current_item == 18) /* MENU_VAR_TIMER_PAGE select a timer page */
      {
        menu_srv_dec_inc_value(item_type, (uint8_t *) & rl_data.menu.rc_timer.current_page);

        menu_rc_timer_write_editable_items();
      }

      break;
    }

    case KP_CURSOR_MIDDLE:
    {
        /* save var and leave menu */
        eeprom_write_var_to_eeprom(EEPROM_ADDRESS(timer[0]), (uint8_t *) &nv_data.timer[0], RL_RC_TIMER_PAGES * sizeof(NV_STRUC_TIMER));

        //menu_selection(MENU_INDEX_MAIN);

        break;
    }

    case KP_RIGHT:
    {
      /* reload var from eeprom and leave menu */
      eeprom_read_var_from_eeprom((uint8_t *) &nv_data.timer[0], EEPROM_ADDRESS(timer[0]), RL_RC_TIMER_PAGES * sizeof(NV_STRUC_TIMER));

      menu_selection(MENU_INDEX_MAIN);

      break;
    }
  }
}


/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/
