/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* this menu shows the last reboot information */
const  PROGMEM char menu_reboots_line_1_4[] = "--.-- --:--";



/* coldstart initialisation of menu page "current values" */
void menu_reboots_coldstart_init (void)
{

}

/* init function of menu page "current values" - called once if menu page has changed */
void menu_reboots_init (void)
{
  uint8_t youngest_index, line;

  lcd_clrscr();

  youngest_index = nv_data.btime.index;

  for(line=0; line<((nv_data.btime.reboot_ctr<4) ? nv_data.btime.reboot_ctr : 4); line++)
  {
      lcd_puts_p(0, line, &menu_reboots_line_1_4[0]);
      lcd_put_two_digit_dec_u8(0, line, nv_data.btime.entry[youngest_index].watch_el[0]);
      lcd_put_two_digit_dec_u8(3, line, nv_data.btime.entry[youngest_index].watch_el[1]);
      lcd_put_two_digit_dec_u8(6, line, nv_data.btime.entry[youngest_index].watch_el[2]);
      lcd_put_two_digit_dec_u8(9, line, nv_data.btime.entry[youngest_index].watch_el[3]);
      lcd_put_two_digit_hex_u8(12, line, nv_data.btime.entry[youngest_index].reset_flag);
      lcd_put_four_digit_dec_u16(15, line, nv_data.btime.entry[youngest_index].line);

      if(youngest_index==0)
      {
        youngest_index = 3;
      }
      else
      {
        youngest_index--;
      }
  }
}

/* regular service function of menu page "current values" - called every second */
void menu_reboots_srv (void)
{
  switch(rl_data.key.bank[KP_INDEX].pressed)
  {
      case KP_CURSOR_LEFT:
      {
          nv_data.btime.reboot_ctr = 0;
          nv_data.btime.index      = 0xFF;

          eeprom_write_var_to_eeprom(EEPROM_ADDRESS(btime), (uint8_t *) &nv_data.btime, sizeof(NV_STRUC_BOOTTIME));

          /* no break */
      }

      case KP_CURSOR_MIDDLE:
      {
          /* clear reboot led */
          REBOOT_LED_OFF;

          break;
      }

      case KP_RIGHT:
      {
          /* leave this menu (escape) */
          menu_selection(MENU_INDEX_MAIN);

          break;
      }
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

