/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* this module treats reception of a remote control with on-off-keying (hs2262 chip ) */


const RC_CODE_LOCATION rc_sys_code[5] = { {0, 6}, {0, 4}, {0, 2}, {0, 0}, {1, 6} }; /* marshalling of syscode 1..5 */
const RC_CODE_LOCATION rc_dev_code[5] = { {1, 4}, {1, 2}, {1, 0}, {2, 6}, {2, 4} }; /* marshalling of devcode A..E */

void     rc_init                  (void)
{
  if(nv_data.misc.properties & RL_PROP_SUPPORT_RC_RCVE)
  {
    rc_rx_init();
  }

  rc_tx_init();
}


uint8_t rc_get_syscode_digit_from_device (uint8_t * rc_code, uint8_t sys_code_digit_nr)
{
  /* get value of system code digit position (possible 0..4) from the remote control system code variable of the assigned blind number */
  uint8_t digit_value;
  uint8_t binary_value;

  uint8_t byte_loc   = rc_sys_code[sys_code_digit_nr].byte_loc;
  uint8_t byte_shift = rc_sys_code[sys_code_digit_nr].byte_shift;
  digit_value        = (*(rc_code+byte_loc) >> byte_shift) & 0x3;

  switch(digit_value)
  {
    case RC_DIGIT_VALUE_FLOATING:
      {binary_value = RC_BINARY_VALUE_FLOATING; break;}
    case RC_DIGIT_VALUE_ZERO:
      {binary_value = RC_BINARY_VALUE_ZERO; break;}
    case RC_DIGIT_VALUE_ONE:
      {binary_value = RC_BINARY_VALUE_ONE; break;}
    default:
      {binary_value = RC_BINARY_VALUE_INVALID; break;}
  }

  return(binary_value);
}

void   rc_set_syscode_digit_to_device (uint8_t * rc_code, uint8_t sys_code_nr, uint8_t sys_code_value)
{
  /* store the value of a given system code digit number (possible 0..4) into the remote control system code variable of the assigned blind number */
  uint8_t digit_value;

  uint8_t byte_loc   = rc_sys_code[sys_code_nr].byte_loc;
  uint8_t byte_shift = rc_sys_code[sys_code_nr].byte_shift;

  switch(sys_code_value)
  {
    case RC_BINARY_VALUE_ZERO:
      {digit_value = RC_DIGIT_VALUE_ZERO; break;}
    case RC_BINARY_VALUE_FLOATING:
      {digit_value = RC_DIGIT_VALUE_FLOATING; break;}
    case RC_BINARY_VALUE_ONE:
      {digit_value = RC_DIGIT_VALUE_ONE; break;}
    default :
      {digit_value = RC_DIGIT_VALUE_INVALID; break;} /* invalid bit combination - must not happen */
  }
  if(digit_value != RC_DIGIT_VALUE_INVALID)
  {
    *(rc_code+byte_loc) = *(rc_code+byte_loc) & ~(RC_DIGIT_VALUE_MASK<<byte_shift);
    *(rc_code+byte_loc) = *(rc_code+byte_loc) |  (digit_value<<byte_shift);
  }
}


uint8_t rc_get_devcode_digit_from_device (uint8_t rc_code_02)
{
  return( rc_code_02 & 0x07 );
}

void   rc_set_devcode_digit_to_device (uint8_t * rc_code_02, uint8_t binary_dev_code)
{
  *rc_code_02 = (*rc_code_02 & 0xF8) + binary_dev_code;
}


/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

