/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

/* sevicing the ad-converter to determine the brightness: therefore two foto resistors are hooked to the adc pins */

void sensor_init (void)
{
    rl_data.sensor.index = 0;

    sensor_start_adc_conversion(); /* initialize first conversion */
}


/* initial start of the ad converter - called during startup */
void sensor_start_adc_conversion (void)
{
  ADMUX = (0x60 | rl_data.sensor.index);  /* AVCC, internal reference voltage */

#if defined CPU_ATMEGA16
  SFIOR  = 0xE0;                        /* free running mode no AC input */
#elif defined CPU_ATMEGA168
  ADCSRB = 0;                           /* free running mode no AC input */
#endif
  ADCSRA = 0xC7;                        /* ADC enable, ADC start conversion, prescaler divion factor #8 */
}

/* check completion of adc cycle and change to the other adc channel */
/* determine the average for every adc brightness sensor and determine the average of both brighness sensors */
void sensor_check_adc_conversion (void)
{
  if(!(ADCSRA & 0x40) )
  {
    /* conversion complete */

    uint8_t result = ADCH;

    /* bring down noises onneighbour channels */
    if( result && (result<5) ) result = 0;

    /* write back the new average sensor value - due to hardware es brighter as less voltage is delivered, so we complement the result */
    rl_data.sensor.single[rl_data.sensor.index] = result;

    if(++rl_data.sensor.index == SENSOR_COUNT)
    {
        rl_data.sensor.index = 0;
    }

    sensor_start_adc_conversion();
  }
}

/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

