/******************************************************************************/
/* Project:   blind control set                                               */
/* Author:    BBC-Hoerer                                                      */
/*            You may use or modify this code for your purpose.               */
/*            BBC-Hoerer is not responsible for consequences using this code. */
/******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "i2cmaster.h"
#include "lcd.h"
#include "_rl_com.h"

//#define SERVICE_MODE_CHANGE_RC_CODE
void  service_mode_init (void)
{
  rl_data.key.service_mode.entered = SERVICE_MODE_OFF;
  rl_data.key.service_mode.button  = 0xFF;
}


void  service_mode_entering_intervall_reset (void)
{
  if( ++rl_data.key.service_mode.intervall_time == SERVICE_MODE_INTERVALL_TIME )
  {
    rl_data.key.service_mode.intervall_time  = 0;
    rl_data.key.service_mode.intervall_count = 0;
  }
}

void  service_mode_check_enter_exit (uint8_t bank, uint8_t rl_number)
{
  static uint8_t reminded_bank = 0xFF; /* set to invalid */

  if(rl_data.key.service_mode.entered == SERVICE_MODE_OFF)
  {
      /* we check if we should enter service mode due to wallbutton up is pressed for SERVICE_MODE_INTERVALL_COUNT within SERVICE_MODE_INTERVALL_TIME */
      if( (reminded_bank == bank) && (bank == WB_UP_INDEX) && (rl_data.key.service_mode.button == rl_number) )
      {
          if(++rl_data.key.service_mode.intervall_count == SERVICE_MODE_INTERVALL_COUNT)
          {
              /* all requirements are fulfilled */

              /* clear reboot led */
              REBOOT_LED_OFF;

              MOTOR_REQ req;
              rl_data.key.service_mode.entered = SERVICE_MODE_IDLE;

              req.opc = MOTOR_OPC_HALT;

              req.rl_number   = rl_number;
              req.runtime     = MOTOR_RUNTIME_FULL;

              motor_request(&req);

              #if defined SERVICE_MODE_CHANGE_RC_CODE  /* we do not allow blind rc code set */
              rl_data.key.service_mode.time_accept_rc_code_change = SERVICE_MODE_CODE_CHANGE_TIME; /* start a time of several seconds within a remote control code change is allowed */
              #else
              rl_data.key.service_mode.time_accept_rc_code_change = 0; /* stop time for changing rc code */
              #endif
          }
      }
      else
      {
          reminded_bank   = bank;

          rl_data.key.service_mode.intervall_time = 0;
          rl_data.key.service_mode.button = rl_number;
      }
  }
  else
  {
      /* we exit the service mode */
      if( (bank == WB_DOWN_INDEX) && (rl_data.key.service_mode.button == rl_number) )
      {
          reminded_bank = 0xFF; /* set to invalid */
          rl_data.alm.shadowing_sm[rl_number].state = (nv_data.d_blind[rl_number].enable != 0) ? ALM_M_AUTO_MODE_INITIALIZED : ALM_M_AUTO_MODE_OFF;
          rl_data.key.service_mode.entered = SERVICE_MODE_OFF;
          rl_data.key.service_mode.button  = 0xFF;
      }
  }
}


uint8_t  service_mode_evaluation  (void)
{
    uint8_t ret_value;

    if /* we are not in service mode */
       (rl_data.key.service_mode.entered != 0)
    {
        if /* we are in a time slot to allow changing of rc code */
           (rl_data.key.service_mode.time_accept_rc_code_change)
        {
            rl_data.key.service_mode.time_accept_rc_code_change = 0; /* stop time for changing rc code */

            #if defined SERVICE_MODE_CHANGE_RC_CODE  /* we do not allow blind rc code set */
            nv_data.motor[rl_data.key.service_mode.button].rc_code[0] = rl_data.rc_rx.user_tel.raw[0];
            nv_data.motor[rl_data.key.service_mode.button].rc_code[1] = (rl_data.rc_rx.user_tel.raw[1] & 0xC0);
            rc_set_devcode_digit_to_device (&nv_data.motor[rl_data.key.service_mode.button].rc_code[1], rl_data.rc_rx.user_tel.device_code);

            eeprom_write_var_to_eeprom(EEPROM_ADDRESS(motor[rl_data.key.service_mode.button].rc_code[0]), (uint8_t *) &nv_data.motor[rl_data.key.service_mode.button].rc_code[0], sizeof(nv_data.motor[rl_data.key.service_mode.button].rc_code[0]));
            #endif
        }
        else
        {
              if(rl_data.rc_rx.user_tel.on_off_code == RC_DIGIT_VALUE_ON_OFF)
              {
                    /* on and off button pressed simulaneousely -> change service mode main chapter */
                    if(rl_data.rc_rx.user_tel.device_code == 0) /* A pressed */
                    {
                      rl_data.key.service_mode.entered = SERVICE_MODE_N_BLIND;
                    }
                    else if(rl_data.rc_rx.user_tel.device_code == 1) /* B pressed */
                    {
                      rl_data.key.service_mode.entered = SERVICE_MODE_D_BLIND;
                    }
                    else if(rl_data.rc_rx.user_tel.device_code == 2) /* C pressed */
                    {
                      rl_data.key.service_mode.entered = SERVICE_MODE_MOTOR;
                    }
                    else if(rl_data.rc_rx.user_tel.device_code == 3) /* D pressed */
                    {
                      rl_data.key.service_mode.entered = SERVICE_MODE_GLOBAL;
                    }
              }
              else
              {
                  uint8_t service_code = (rl_data.rc_rx.user_tel.device_code<<1);

                  if(rl_data.rc_rx.user_tel.on_off_code == RC_DIGIT_VALUE_OFF)
                  {
                      service_code |= 1;
                  }

                  uint8_t rl_count = rl_data.key.service_mode.button;

                  if(rl_data.key.service_mode.entered == SERVICE_MODE_N_BLIND)
                  {
                      uint8_t page = rl_data.alm.current_page;
                      switch( service_code )
                      {
                          case 0: /* device code A and UP --> include window into night blind */
                          {
                            nv_data.n_blind_window.included |= _BV(rl_count);
                            break;
                          }
                          case 1: /* device code A and DOWN --> exclude window into night blind */
                          {
                            nv_data.n_blind_window.included &= ~_BV(rl_count);
                            break;
                          }
                          case 2: /* device code B and UP --> night blind up time */
                          {
                            rv3029_get_current_hour_minute((uint8_t *)&nv_data.n_blind[page].hour_up, (uint8_t *)&nv_data.n_blind[page].minute_up);
                            break;
                          }
                          case 3: /* device code B and DOWN --> night blind down time */
                          {
                            rv3029_get_current_hour_minute((uint8_t *)&nv_data.n_blind[page].hour_down, (uint8_t *)&nv_data.n_blind[page].minute_down); // @suppress("Invalid arguments")
                            break;
                          }
                          case 4: /* device code C and UP --> window has a lightbow */
                          {
                            nv_data.adv.has_lightbow |= _BV(rl_count);
                            break;
                          }
                          case 5: /* device code C and DOWN --> window does not have a lightbow */
                          {
                            nv_data.adv.has_lightbow &= ~_BV(rl_count);
                            break;
                          }
                          case 6: /* device code D and UP --> unused */
                          {
                            break;
                          }
                          case 7: /* device code D and DOWN --> unused */
                          {
                            break;
                          }
                          default:
                          {
                            break;
                          }
                      }
                      eeprom_write_var_to_eeprom(EEPROM_ADDRESS(n_blind[page]), (uint8_t *) &nv_data.n_blind[page], sizeof(NV_STRUC_N_BLIND));
                      eeprom_write_var_to_eeprom(EEPROM_ADDRESS(n_blind_window), (uint8_t *) &nv_data.n_blind_window, sizeof(NV_STRUC_N_BLIND_WINDOW));
                      eeprom_write_var_to_eeprom(EEPROM_ADDRESS(adv), (uint8_t *) &nv_data.adv, sizeof(NV_STRUC_ADV));

                  }
                  else if(rl_data.key.service_mode.entered == SERVICE_MODE_D_BLIND)
                  {
                      NV_STRUC_D_BLIND * current_d_blind = (NV_STRUC_D_BLIND *) &nv_data.d_blind[rl_count];

                      switch( service_code )
                      {
                          case 0: /* device code A and UP --> activate daylight shadowing for the window */
                          {
                            current_d_blind->enable = 1;
                            break;
                          }
                          case 1: /* device code A and DOWN --> deactivate daylight shadowing for the window */
                          {
                            current_d_blind->enable = 0;
                            break;
                          }
                          case 2: /* device code B and UP --> daylight shadowing: current time for driving the blind up */
                          {
                            rv3029_get_current_hour_minute((uint8_t *) &current_d_blind->hour_up, (uint8_t *) &current_d_blind->minute_up);
                            break;
                          }
                          case 3: /* device code B and DOWN --> daylight shadowing: current time for driving the blind down */
                          {
                            rv3029_get_current_hour_minute((uint8_t *) &current_d_blind->hour_down, (uint8_t *) &current_d_blind->minute_down);
                            break;
                          }
                          case 4: /* device code C and UP --> daylight shadowing: current sensor value is taken for driving the blind up */
                          {
                            current_d_blind->sensor_up = rl_data.sensor.single[rl_count];
                            break;
                          }
                          case 5: /* device code C and DOWN --> daylight shadowing: current sensor value is taken for driving the blind down */
                          {
                            current_d_blind->sensor_down = rl_data.sensor.single[rl_count];
                            break;
                          }
                          case 6: /* device code D and UP --> increment t_hyst */
                          {
                            if(current_d_blind->time_hyst<59) current_d_blind->time_hyst++;
                            break;
                          }
                          case 7: /* device code D and DOWN --> deccrement t_hyst */
                          {
                            if(current_d_blind->time_hyst>0) current_d_blind->time_hyst--;
                            break;
                          }
                          default:
                          {
                            break;
                          }
                      }
                      eeprom_write_var_to_eeprom(EEPROM_ADDRESS(d_blind[rl_count]), (uint8_t *) &nv_data.d_blind[rl_count], sizeof(NV_STRUC_D_BLIND));
                  }
                  else if(rl_data.key.service_mode.entered == SERVICE_MODE_MOTOR)
                  {
                      MOTOR_REQ req;

                      switch( service_code )
                      {
                          case 0: /* device code A and UP --> unused */
                          {
                            break;
                          }
                          case 1: /* device code A and DOWN --> unused */
                          {
                            break;
                          }
                          case 2: /* device code B and UP --> unused */
                          {
                            break;
                          }
                          case 3: /* device code B and DOWN --> unused */
                          {
                            break;
                          }
                          case 4: /* device code C and UP --> drive motor up */
                          {
                            req.opc     = MOTOR_OPC_UP;
                            req.rl_number = rl_count;
                            req.runtime   = 2; /* stop after 2 seconds */
                            if /* first call results in state change - start service runtime */
                              ( motor_request(&req)  )
                            {
                              rl_data.motor.service_runtime = 0;
                            }
                            break;
                          }
                          case 5: /* device code C and DOWN  --> drive motor down */
                          {
                            req.opc     = MOTOR_OPC_DOWN;
                            req.rl_number = rl_count;
                            req.runtime   = 2; /* stop after 2 seconds */
                            motor_request(&req);
                            break;
                          }
                          case 6: /* device code D and UP  --> jarolift programming mode for limit stop */
                          {
                            req.opc     = MOTOR_OPC_PGM;
                            req.rl_number = rl_count;
                            req.runtime   = 2; /* stop after 2 seconds */
                            motor_request(&req);
                            break;
                          }
                          case 7: /* device code D and DOWN --> store motor runtime  */
                          {
                            if(rl_data.motor.service_runtime>2)
                            {
                                nv_data.motor[rl_count].runtime = rl_data.motor.service_runtime;
                                eeprom_write_var_to_eeprom(EEPROM_ADDRESS(motor[rl_count].runtime), (uint8_t *) &nv_data.motor[rl_count].runtime, sizeof(nv_data.motor[rl_count].runtime));
                            }
                            break;
                          }
                      }
                  }
                  else if(rl_data.key.service_mode.entered == SERVICE_MODE_GLOBAL)
                  {
                      switch( service_code )
                      {
                          case 0: /* device code A and UP --> night blind automatic switched on */
                          {
                            nv_data.misc.properties |= RL_PROP_SUPPORT_N_BLIND;
                            break;
                          }
                          case 1: /* device code A and DOWN --> night blind automatic switched off */
                          {
                            nv_data.misc.properties &= ~RL_PROP_SUPPORT_N_BLIND;
                            break;
                          }
                          case 2: /* device code B and UP --> remote control reception in non service mode switched on */
                          {
                            nv_data.misc.properties |= RL_PROP_SUPPORT_RC_RCVE;
                            break;
                          }
                          case 3: /* device code B and DOWN --> remote control reception in non service mode switched off */
                          {
                            nv_data.misc.properties &= ~RL_PROP_SUPPORT_RC_RCVE;
                            break;
                          }
                          case 4: /* device code C and UP --> advent mode switched on */
                          {
                            nv_data.misc.properties |= RL_PROP_SUPPORT_ADVENT;
                            break;
                          }
                          case 5: /* device code C and DOWN --> advent mode switched off */
                          {
                            nv_data.misc.properties &= ~RL_PROP_SUPPORT_ADVENT;
                            break;
                          }
                          case 6: /* device code D and UP --> unused */
                          {
                            break;
                          }
                          case 7: /* device code D and DOWN --> unused */
                          {
                            break;
                          }
                          default:
                          {
                            break;
                          }
                      }
                      eeprom_write_var_to_eeprom(EEPROM_ADDRESS(misc), (uint8_t *) &nv_data.misc, sizeof(NV_STRUC_MISC));
                  }
              }
        }
        ret_value = SERVICE_MODE_EVAL_DONE;
    }
    else
    {
        ret_value = SERVICE_MODE_EVAL_NOT_DONE;
    }

    return(ret_value);
}


/************************************************************************/
/* copyright: BBC-Hoerer 2016-2018                                      */
/************************************************************************/

